package com.gic.enterprise.web.controller.wm;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.constant.MallModeEnum;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.enterprise.service.WmStoreSyncLogApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.wm.WeimobStoreClerkLogVO;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.dto.ClerkDTO;
import com.gic.store.dto.StoreBusinessTimeDTO;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.service.ClerkApiService;
import com.gic.store.service.StoreApiService;
import com.gic.weimob.api.dto.WeimobPhysicalStoreDTO;
import com.gic.weimob.api.dto.WeimobStoreAccountDTO;
import com.gic.weimob.api.service.WeimobStoreSiteService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.redisson.api.RMap;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * 同步门店to微盟
 * @ClassName: WmStoreSyncController 
 * @Description:  
 * @author guojuxing 
 * @date 2020/5/8 11:19 AM 
 */
@RestController
@RequestMapping("/wm-store-sync")
public class WmStoreSyncController {
    private static final Logger LOGGER = LogManager.getLogger(WmStoreSyncController.class);

    @Autowired
    private WmStoreApiService wmStoreApiService;
    @Autowired
    private WeimobStoreSiteService weimobStoreSiteService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private WmStoreSyncLogApiService wmStoreSyncLogApiService;
    @Autowired
    private ClerkApiService clerkApiService;


    private static final String REDIS_KEY = "enterprise:weimob4:init:";
    /**
     * 分店导入redis日志key
     */
    private static final String STORE_BRANCH_REDIS_KEY = "enterprise:weimob4:init:branch:";


    /**
     *
     * @param request
     * @param response
     * @throws IOException
     */
    @RequestMapping("wm-store-import-template")
    public void wmStoreImportTemplate(HttpServletRequest request, HttpServletResponse response)
            throws IOException {
        ClassPathResource res = new ClassPathResource("exceltemplate/wm_store_import_template.xls");
        String userAgent = request.getHeader("User-Agent").toLowerCase();
        String fileName = "门店同步模板.xls";
        if (userAgent.contains("firefox")) {
            byte[] bytesName = fileName.getBytes("UTF-8");
            fileName = new String(bytesName, "ISO-8859-1");
            response.setHeader("content-disposition", "attachment;fileName=" + fileName);
        } else {
            response.setHeader("content-disposition", "attachment;fileName=" + URLEncoder.encode(fileName, "UTF-8"));
        }

        InputStream is = res.getInputStream();
        OutputStream out = response.getOutputStream();
        int bytes = 0;
        byte[] bufferOut = new byte[1024];
        while ((bytes = is.read(bufferOut)) != -1) {
            out.write(bufferOut, 0, bytes);
        }

        out.close();
        is.close();
    }

    /**
     * 总店-分店同步
     * @Title: syncHeadStore 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 店铺逐渐ID
     * @param headStoreId 总店ID
     * @param storeIds  分店IDs
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("sync-head-store")
    public RestResponse syncHeadStore(Integer wmMallStoreId, Integer headStoreId, String storeIds) {
        ServiceResponse<WmStoreDTO> wmStoreResponse = wmStoreApiService.getWmStoreByWmMallStoreId(wmMallStoreId);
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if (wmStoreResponse.isSuccess()) {
            WmStoreDTO wmStoreDTO = wmStoreResponse.getResult();
            //是否智慧零售模式
            Integer mode = wmStoreDTO.getMallMode();
            if (mode.intValue() != MallModeEnum.SMART_RETAIL.getCode()) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "非" + MallModeEnum.SMART_RETAIL.getMsg() + "不能同步门店");
            }
            //查询总店
            ServiceResponse<StoreDTO> storeResponse = storeApiService.getStoreById(enterpriseId, headStoreId);
            if (!storeResponse.isSuccess()) {
                return RestResponse.failure(storeResponse.getCode(), storeResponse.getMessage());
            }
            //装载总店数据为微盟数据
            StoreDTO storeDTO = storeResponse.getResult();
            WeimobPhysicalStoreDTO weimobPhysicalStoreDTO = getWeimobPhsicalStore(storeDTO);
            if (weimobPhysicalStoreDTO == null) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "总店不存在或者未启用");
            }
            //分店
            List<WeimobPhysicalStoreDTO> subStoreList = new ArrayList<>();
            if (StringUtils.isNotBlank(storeIds)) {
                List<StoreDTO> storeList = getStoreList(storeIds);
                //循环装载微盟门店数据
                storeList.forEach(tempStore -> subStoreList.add(getWeimobPhsicalStore(tempStore)));
            }
            ServiceResponse<Void> wmResult = weimobStoreSiteService
                    .addWeimobVirtualStore(wmMallStoreId, enterpriseId, weimobPhysicalStoreDTO, subStoreList);
            if (wmResult.isSuccess()) {
                wmStoreSyncLogApiService.saveWmStoreSyncLog(Arrays.asList(headStoreId), wmMallStoreId, 1, enterpriseId);
                //todo 同步导购
                return RestResponse.success();
            }
            return RestResponse.failure(wmResult.getCode(), wmResult.getMessage());
        }
        return RestResponse.failure(wmStoreResponse.getCode(), wmStoreResponse.getMessage());
    }

    /**
     * 已同步的门店列表
     * @Title: listWmStoreLog 
     * @Description:
     * @author guojuxing
     * @param search 门店code/门店名称
     * @param type 1：总店 2：一对一
     * @param wmMallStoreId 店铺主键ID
     * @param currentPage
     * @param pageSize 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("list-wm-has-sync-store")
    @ResponseBody
    public RestResponse listWmStoreLog(String search, Integer type, Integer wmMallStoreId,
                                 @RequestParam(defaultValue = "1") int currentPage, @RequestParam(defaultValue = "20") int pageSize) {
        return ResultControllerUtils.commonResult(wmStoreSyncLogApiService
                .pageWmStoreSyncLog(search, type, wmMallStoreId, currentPage, pageSize));
    }

    /**
     * 解除门店同步日志记录
     * @param storeId
     * @return
     */
    @RequestMapping("delete-logic")
    @ResponseBody
    public RestResponse deleteLogic(Integer storeId, Integer wmMallStoreId) {
        if (storeId == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数为空");
        }
        wmStoreSyncLogApiService.deleteLogic(storeId, wmMallStoreId);

        ServiceResponse<Void> weimobResult = weimobStoreSiteService.deleteStoreRel(wmMallStoreId, storeId);
        LOGGER.info("微盟中间件删除门店关联关系结果:{}", JSON.toJSONString(weimobResult));
        if (!weimobResult.isSuccess()) {
            return RestResponse.failure(weimobResult.getCode(), weimobResult.getMessage());
        }
        return RestResponse.success();
    }

    /**
     * 门店同步错误日志
     * @param wmMallStoreId 店铺ID
     * @param logType -- 非必填，null时取全部，STORE取门店,CLERK取导购
     * @param pageSize
     * @param currentPage
     * @param search
     * @return
     */
    @RequestMapping("list-store-clerk-error-log")
    @ResponseBody
    public RestResponse listStoreClerkErrorLog(Integer wmMallStoreId, String logType,
                                               @RequestParam(defaultValue="20") int pageSize,
                                               @RequestParam(defaultValue="1")int currentPage, String search) {
        return ResultControllerUtils.commonPageResult(weimobStoreSiteService
                .getStoreClerkLog(wmMallStoreId, logType, currentPage, pageSize, search), WeimobStoreClerkLogVO.class);
    }

    /**
     * 门店错误日志-不处理
     * @Title: deleteStoreClerkErrorLog 
     * @Description:
     * @author guojuxing
     * @param id 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("delete-store-clerk-error-log")
    @ResponseBody
    public RestResponse deleteStoreClerkErrorLog(String id) {
        return ResultControllerUtils.commonResult(weimobStoreSiteService.deleteStoreClerkErrorlog(Long.valueOf(id)));
    }

    /**
     * 一对一门店同步
     * @param wmMallStoreId
     * @param storeIds
     * @return
     */
    @RequestMapping("/weimob-init-store")
    @ResponseBody
    public RestResponse weimobInitStore(Integer wmMallStoreId, String storeIds) {
        if (StringUtils.isNotBlank(storeIds)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数为空");
        }
        ServiceResponse<WmStoreDTO> wmStoreResponse = wmStoreApiService.getWmStoreByWmMallStoreId(wmMallStoreId);
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if (wmStoreResponse.isSuccess()) {
            WmStoreDTO wmStoreDTO = wmStoreResponse.getResult();
            //是否智慧零售模式
            Integer mode = wmStoreDTO.getMallMode();
            if (mode.intValue() != MallModeEnum.SMART_RETAIL.getCode()) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "非" + MallModeEnum.SMART_RETAIL.getMsg() + "不能同步门店");
            }
            List<WeimobPhysicalStoreDTO> physicalStoreList = new ArrayList<>();
            List<StoreDTO> storeList = getStoreList(storeIds);
            storeList.forEach(store -> physicalStoreList.add(getWeimobPhsicalStore(store)));
            ServiceResponse<List<Integer>> wmResult = weimobStoreSiteService
                    .addWeimobPhysicalStoreBatch(wmMallStoreId, enterpriseId, physicalStoreList);
            LOGGER.info("实体门店批量到微盟结果：{}", JSON.toJSONString(wmResult));
            if (!wmResult.isSuccess()) {
                return RestResponse.failure(wmResult.getCode(), wmResult.getMessage());
            }
            //todo 导购同步
        }
        return RestResponse.success();
    }

    /**
     * 一对一门店导入同步日志
     * @return
     */
    @RequestMapping("/weimob-init-error-download-process")
    @ResponseBody
    public RestResponse storeImportTemplateDownload() {
        return getExportProcess(REDIS_KEY);
    }

    /**
     * 分店导入同步日志
     * @return
     */
    @RequestMapping("/weimob-init-branch-error-download-process")
    @ResponseBody
    public RestResponse storeBranchExportProcess() {
        return getExportProcess(STORE_BRANCH_REDIS_KEY);
    }

    private WeimobPhysicalStoreDTO getWeimobPhsicalStore(StoreDTO storeDTO) {
        if (storeDTO.getStatus().intValue() == 1) {
            WeimobPhysicalStoreDTO weimobPhysicalStoreDTO = new WeimobPhysicalStoreDTO();
            weimobPhysicalStoreDTO.setStoreId(storeDTO.getStoreId());
            weimobPhysicalStoreDTO.setStoreNumber(storeDTO.getStoreCode());
            weimobPhysicalStoreDTO.setStoreName(storeDTO.getStoreName());
            //经纬度
            if (StringUtils.isNotBlank(storeDTO.getLongitude())) {
                weimobPhysicalStoreDTO.setLongitude(new BigDecimal(storeDTO.getLongitude()));
            }
            if (StringUtils.isNotBlank(storeDTO.getLatitude())) {
                weimobPhysicalStoreDTO.setLatitude(new BigDecimal(storeDTO.getLatitude()));
            }
            //省市区名称
            String fullArea = StringUtils.isNotBlank(storeDTO.getProvinces()) ?
                    storeDTO.getProvinces().replaceAll("//", "") : "";
            weimobPhysicalStoreDTO.setAddress(fullArea + storeDTO.getAddress());
            StringBuilder businessTime = new StringBuilder();
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH:mm");
            List<StoreBusinessTimeDTO> businessTimeList = storeDTO.getBusinessTimeList();
            if (CollectionUtils.isNotEmpty(businessTimeList)) {
                businessTimeList.stream().map(e -> {
                    String businessHours = simpleDateFormat.format(e.getOpenTime()) + "-" + simpleDateFormat.format(e.getCloseTime());
                    return e.getWeekdayShow() + businessHours;
                }).forEach(businessTime::append);
            }
            weimobPhysicalStoreDTO.setBussinessHours(businessTime.toString());

            WeimobStoreAccountDTO accountDTO = new WeimobStoreAccountDTO();
            ServiceResponse<ClerkDTO> clerkResponse = clerkApiService.getHeadClerk(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
            ClerkDTO headClerk = null;
            if (clerkResponse.isSuccess()) {
                headClerk = clerkResponse.getResult();
                if (headClerk != null) {
                    accountDTO.setManagerLoginAccount(headClerk.getPhoneNumber());
                    accountDTO.setManagerName(headClerk.getClerkName());
                }
            }
            if (StringUtils.isNotBlank(storeDTO.getConactsPhone())) {
                weimobPhysicalStoreDTO.setStoreTel(storeDTO.getConactsPhone());
            } else {
                //店长的号码
                if (headClerk != null) {
                    weimobPhysicalStoreDTO.setStoreTel(headClerk.getPhoneNumber());
                }
            }
            weimobPhysicalStoreDTO.setAccountVo(accountDTO);
            return weimobPhysicalStoreDTO;
        }
        return null;
    }

    private List<StoreDTO> getStoreList(String storeIds) {
        String[] subStoreIdArr = storeIds.split(",");
        //一次性查询分店列表数据
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setStoreIds(storeIds);
        ServiceResponse<Page<StoreDTO>> subStoreResponse = storeApiService.listStore(storeSearchDTO, 0, subStoreIdArr.length);
        if (subStoreResponse.isSuccess()) {
            Page<StoreDTO> page = subStoreResponse.getResult();
            if (page != null) {
                List<StoreDTO> storeList = page.getResult();
                return storeList;
            }
        }
        return Collections.emptyList();
    }

    /**
     * Excel导入进度查询
     * @param key
     * @return
     */
    private RestResponse getExportProcess(String key) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RMap<String, String> rMap = redisClient.getMap(key + enterpriseId);
        if (rMap.size() > 0) {
            String total = rMap.get("一对一门店同步总数");
            if (org.apache.commons.lang3.StringUtils.isBlank(total)) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不存在同步任务");
            }
            Integer totalCount = Integer.parseInt(total);
            DecimalFormat df = new DecimalFormat("0.0000");
            return RestResponse.success(df.format((rMap.size() - 1) / totalCount));
        }
        return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不存在同步任务");
    }
}
